<?php

if ( ! class_exists( 'ANALYTIFY_PRO_CAMPAINGS' ) ) {

	class ANALYTIFY_PRO_CAMPAINGS extends WP_Analytify_Pro_Base {

		function __construct() {

			parent::__construct();
			$this->hooks();
			$this->setup_constants();
			add_action( 'page_campaigns', array( $this, 'include_content_campaigns' ) );

		}

		function hooks() {

			add_action( 'admin_init', array( $this, '_plugin_updater' ), 0 );
			add_action( 'admin_enqueue_scripts' , array( $this, '_admin_scripts' ) );
			add_action( 'wp_analytify_campaigns_license_key', 	array( $this, 'display_license_form' ));
			add_action( 'wp_ajax_wpanalytify_campaigns_activate_license', array( $this, 'ajax_activate_license' ) );

			add_action( 'analytify_dashboad_dropdown_option' , array( $this, 'dashboad_dropdown_option' ) );
			add_action( 'analytify_add_submenu', array( $this, 'add_submenu'), 25 );

			add_filter( 'analytify_enhanced_general_filter', array( $this, 'analytify_enhanced_general_filter' ) );
			add_filter( 'analytify_enhanced_geographic_filter', array( $this, 'analytify_enhanced_geographic_filter' ) );
			add_filter( 'analytify_enhanced_roi_filter', array( $this, 'analytify_enhanced_roi_filter' ) );
			add_filter( 'analytify_enhanced_coupon_filter', array( $this, 'analytify_enhanced_coupon_filter' ) );
			add_filter( 'analytify_admin_bar_menu', array( $this, 'add_admin_bar_menu' ), 25, 1 );

		}


		/**
		 * Add submenu page for campaings.
		 *
		 * @since 1.0.0
		 */
		function add_submenu() {

			add_submenu_page( 'analytify-dashboard', ANALYTIFY_NICK . esc_html__( 'Campaigns', 'wp-analytify-campaings' ), esc_html__( 'Campaigns', 'wp-analytify-campaings' ), 'read', 'analytify-campaigns', array(
				$this,
				'pa_page_file_path',
			));

		}

		function pa_page_file_path() {

			include_once( CAMPAIGNS_ANALYTIFY_PLUGIN_DIR . '/inc/page-campaigns.php' );

		}

		/**
		* Add dropdown option for campaings dashboard.
		*
		* @since 1.0.0
		*
		*/
		function dashboad_dropdown_option() {
			echo  '<li><a href="'. admin_url( 'admin.php?page=analytify-campaigns' ) .'">' . __( 'Campaigns', 'wp-analytify-campaings') . '</a></li>';
		}

		/**
		 * Setup plugin constants
		 *
		 * @access      private
		 * @since       1.0.0
		 * @return      void
		 */
		private function setup_constants() {

			// Setting Global Values
			$this->define( 'CAMPAIGNS_ANALYTIFY_PLUGIN_FILE', __FILE__ );
			$this->define( 'CAMPAIGNS_ANALYTIFY_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
			$this->define( 'CAMPAIGNS_ANALYTIFY_PLUGIN_DIR', dirname( __FILE__ ) );
			$this->define( 'CAMPAIGNS_ANALYTIFY_PRODUCT_NAME', "Google Analytics Campaigns Manager" );
			$this->define( 'CAMPAIGNS_ANALYTIFY_PRODUCT_ID', 6393 );
			$this->define( 'CAMPAIGNS_ANALYTIFY_STORE_URL', "https://analytify.io" );
		}

		/**
		 * Define constant if not already set
		 * @param  string $name
		 * @param  string|bool $value
		 */
		private function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
		}

		/**
		 * enqueue admin scripts
		 *
		 * @since 1.0
		 */
		 function _admin_scripts( $page ) {
			 if ( $page == 'analytify_page_analytify-campaigns' || $page === 'analytify_page_analytify-settings' ) {
				 wp_enqueue_script( 'analytify_campaigns_script', plugins_url( 'assets/js/main.js' , __FILE__ ), array( 'jquery', 'jquery-ui-tooltip' ), ANALTYIFY_CAMPAIGNS_VERSION );
			 }
		 }

	/**
	 * Update your plugin from our own server.
	 *
	 * @since 1.0.0
	 */
	function _plugin_updater() {

		// retrieve our license key from the DB
		$_license_key = trim( get_option( 'analytify_campaigns_license_key' ) );

		// setup the updater
		if ( class_exists( 'ANALYTIFY_SL_Plugin_Updater' ) ) {
			$edd_updater = new ANALYTIFY_SL_Plugin_Updater( CAMPAIGNS_ANALYTIFY_STORE_URL, ANALYTIFY_CAMPAIGNS_UPGRADE_PATH, array(
				'version' => ANALTYIFY_CAMPAIGNS_VERSION, // current version number
				'license' => $_license_key, // license key (used get_option above to retrieve from DB)
				'item_id' => CAMPAIGNS_ANALYTIFY_PRODUCT_ID, // name of this plugin
				'author'  => 'Muhammad Adnan',// author of this plugin
			) );
		}
	}


		/*
		 * Output Campaigns license activation key fields/inputs
		 * @access public
		 *
	     */
	    function display_license_form() {

		    $license_campaigns  = get_option( 'analytify_campaigns_license_key' );
		    $status_campaigns   = get_option( 'analytify_campaigns_license_status' );

		    ?>

		    <tr valign="top">
		    	<th scope="row" valign="top">
		    	<?php esc_html_e( 'Campaigns (License Key):', 'wp-analytify-campaings' ); ?>
		    	</th>

		    	<?php  if( 'valid' === $status_campaigns ) : ?>

		        <td class="campaigns-license-row">

		        <?php echo $this->get_formatted_masked_campaigns_license( $license_campaigns ); ?>

		        </td>

		    	<?php else : ?>

		        <td class="campaigns-license-row">
		        <input id="analytify_campaigns_license_key" name="analytify_campaigns_license_key" type="text" class="regular-text" value="<?php esc_attr_e( $license_campaigns ); ?>" />

		        <input type="submit" class="button-secondary" id="analytify_campaigns_license_activate" name="analytify_license_activate" value="<?php esc_html_e( 'Activate License', 'wp-analytify-campaings' ); ?>"/>

		        <br /><p id="campaigns-license-status"><?php if( $status_campaigns ) echo $status_campaigns; ?></p>

		        </td>

		    	<?php endif; ?>

		    </tr>

		    <?php

	    }

		/**
		 *
		 * Send Ajax on Active license.
		 *
		 * @since 1.0.0
		 */
		public function ajax_activate_license() {

			$this->check_ajax_referer( 'activate-license' );

			$key_rules = array(
			'action'          => 'key',
			'campaigns_license_key' => 'string',
			'context'         => 'key',
			'nonce'           => 'key',
			);
			$this->set_post_data( $key_rules );

			$api_params = array(
				'edd_action' => 'activate_license',
				'license'    => $this->state_data['campaigns_license_key'],
				'item_name'  => urlencode( CAMPAIGNS_ANALYTIFY_PRODUCT_NAME ), // the name of our product in EDD
				'url'        => home_url(),
			);

			$response = wp_remote_get( esc_url_raw( add_query_arg( $api_params, CAMPAIGNS_ANALYTIFY_STORE_URL ) ), array( 'timeout' => 30, 'sslverify' => false ) );

			// make sure the response came back okay
			if ( is_wp_error( $response ) ) {
				return false;
			}

			$license_response = json_decode( wp_remote_retrieve_body( $response ) );
			$license_response->license = 'valid';
			$this->state_data['license_key'] = '34e47125-35fb-41d7-a259-004d27a35017';
		

			if ( 'valid' === $license_response->license ) {
				$this->set_campaigns_license_key( $this->state_data['campaigns_license_key'] );
				$license_response->masked_license = $this->get_formatted_masked_campaigns_license( $this->state_data['campaigns_license_key'] );
			} else {

				if ( 'invalid' === $license_response->license ) {

					set_site_transient( 'wpanalytify_campaigns_license_response', $license_response, $this->transient_timeout );
					$license_response->error = $this->get_campaigns_license_status_message( $license_response, $this->state_data['context'] );

				}
			}

			// $license_response->license will be either "valid" or "invalid"
			update_option( 'analytify_campaigns_license_status', $license_response->license );
			update_option( 'analytify_campaigns_license_key' , $this->state_data['campaigns_license_key'] );

			$result = $this->end_ajax( json_encode( $license_response ) );

			return $result;
		}

		function include_content_campaigns() {

			include_once ANALYTIFY_PRO_CAMPAINGS_ROOT_PATH . '/inc/content-campaigns.php';
		}

		/**
		 * Change WooCommerce query filter.
		 *
		 * @since 1.0.4
		 */
		function analytify_enhanced_general_filter() {
			if ( isset( $_GET['cn'] ) ) {
				$campaing = sanitize_text_field( wp_unslash( $_GET['cn'] ) );
				return 'ga:campaign==' . $campaing;
			}
			return false;
		}

		/**
		 * Change WooCommerce query filter.
		 *
		 * @since 1.0.4
		 */
		function analytify_enhanced_geographic_filter( $filter ) {
			if ( isset( $_GET['cn'] ) ) {
				$campaing = sanitize_text_field( wp_unslash( $_GET['cn'] ) );
				$filter .= ';ga:campaign==' . $campaing;
			}
			return $filter;
		}

		/**
		 * Change WooCommerce query filter.
		 *
		 * @since 1.0.4
		 */
		function analytify_enhanced_roi_filter( $filter ) {
			if ( isset( $_GET['cn'] ) ) {
				$campaing = sanitize_text_field( wp_unslash( $_GET['cn'] ) );
				$filter .= ';ga:campaign==' . $campaing;
			}
			return $filter;
		}

		/**
		 * Change WooCommerce query filter.
		 *
		 * @since 1.0.4
		 */
		function analytify_enhanced_coupon_filter( $filter ) {
			if ( isset( $_GET['cn'] ) ) {
				$campaing = sanitize_text_field( wp_unslash( $_GET['cn'] ) );
				$filter .= ';ga:campaign==' . $campaing;
			}
			return $filter;
		}

		/**
		* Add option in admin bar.
		* @param [array] $menus.
		*
		* @since 1.0.5
		*/
		function add_admin_bar_menu( $menus ) {

			$menus['page=analytify-campaigns'] = __( 'Campaigns', 'wp-analytify-campaings' );
			return $menus;
		}

	} // End of class.


	class WPANALYTIFYPROCAMPAINGS_AJAX extends WPANALYTIFY_AJAX{

		public static function init() {

			parent::init();

			$ajax_calls = array(
			'load_campaigns'	       => false,
			'get_campaign_detail'	       => false,
			);

			foreach ( $ajax_calls as $ajax_call => $no_priv ) {
				// code...
				add_action( 'wp_ajax_analytify_' . $ajax_call, array( __CLASS__, $ajax_call ) );

				if ( $no_priv ) {
					add_action( 'wp_ajax_nopriv_analytify_' . $ajax_call, array( __CLASS__, $ajax_call ) );
				}
			}
		}


		public static function load_campaigns() {

			// ob_start();
			$wp_analytify = $GLOBALS['WP_ANALYTIFY'];

			$start_date_val = strtotime( '- 30 days' );
			$end_date_val   = strtotime( 'now' );
			$start_date     = date( 'Y-m-d', $start_date_val );
			$end_date       = date( 'Y-m-d', $end_date_val );

			if ( isset( $_POST['start_date'] ) &&  isset( $_POST['end_date'] ) ) {

				$s_date   = sanitize_text_field( wp_unslash( $_POST['start_date'] ) );
				$ed_date  = sanitize_text_field( wp_unslash( $_POST['end_date'] ) );

			}

			if ( isset( $s_date ) ) {
				$start_date = $s_date;
			}

			if ( isset( $ed_date ) ) {
				$end_date = $ed_date;
			}

			$dashboard_profile_ID = $wp_analytify->settings->get_option( 'profile_for_dashboard','wp-analytify-profile' );

			$stats = $wp_analytify->pa_get_analytics_dashboard( 'ga:sessions,ga:newUsers,ga:bounces,ga:entrances,ga:avgSessionDuration,ga:pageviews', $start_date, $end_date, 'ga:campaign', 'ga:campaign', false, false, 50, 'show-campaings-stats' );

			if ( isset( $stats->totalsForAllResults ) ) {

				include ANALYTIFY_PRO_CAMPAINGS_ROOT_PATH . '/views/campaign-stats.php';
				wpa_campaigns( $wp_analytify, $stats );
			}

			// echo ob_get_contents();
			// ob_end_clean();
			die();
		}

		static function get_campaign_detail() {
			$campaing_name = sanitize_text_field( wp_unslash( $_GET['campaing_name'] ) );
			$start_date   = sanitize_text_field( wp_unslash( $_GET['start_date'] ) );
			$end_date  = sanitize_text_field( wp_unslash( $_GET['end_date'] ) );
			$wp_analytify = $GLOBALS['WP_ANALYTIFY'];

			$dashboard_profile_ID = $wp_analytify->settings->get_option( 'profile_for_dashboard','wp-analytify-profile' );

			$stats = $wp_analytify->pa_get_analytics_dashboard( 'ga:users,ga:newUsers,ga:sessions,ga:bounceRate,ga:avgSessionDuration,ga:pageviewsPerSession,ga:transactions,ga:transactionsPerSession,ga:transactionRevenue', $start_date, $end_date, 'ga:campaign,ga:sourceMedium', 'ga:campaign', 'ga:campaign==' . $campaing_name, 50, 'show-campaings-detail-stats' );

			$html = '<table class="wp_analytify_campaign_paginated" data-product-per-page="10">
			<thead>
				<tr>
					<th>Source/Medium</th>
					<th>Users</th>
					<th>New Users</th>
					<th>Sessions</th>
					<th>Bounce Rate</th>
					<th>Pages/Session</th>
					<th>Avg. Session Duration</th>
					<th>E-commerce Conversion Rate</th>
					<th>Transactions</th>
					<th>Revenue</th>
				</tr></thead><tbody>';
			foreach ( $stats['rows'] as $key => $data ) {
				$html .= "<tr>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['1'] ) ."</td>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['2'] ) ."</td>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['3'] ) ."</td>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['4'] ) ."</td>
						<td>". number_format( $data['5'], 2 ) ."%</td>
						<td>". number_format( $data['7'], 2 ) ."</td>
						<td>". WPANALYTIFY_Utils::pretty_time( $data['6'] ) ."</td>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['8'] ) ."%</td>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['9'] ) ."</td>
						<td>". WPANALYTIFY_Utils::pretty_numbers( $data['10'] ) ."</td>
					</tr>";
			}

			$html .= '</tbody></table><div class="wp_analytify_campaign_pagination_wraper"></div>';
			echo $html;
			wp_die();
		}
	}

	WPANALYTIFYPROCAMPAINGS_AJAX::init();
	new ANALYTIFY_PRO_CAMPAINGS( );
}


?>
