<?php

/**
* Plugin Name: Analytify for Easy Digital Downloads
* Plugin URI: https://analytify.io/enhanced-ecommerce-google-analytics-for-edd/
* Description: This Add-on will track the e-commerce behaviour of users, digital Sales, Transactions and Revenues in Google Analytics and Displays Stats in a unique and intiutive way which is very understandable even for non-technical WordPress users.
* Version: 1.1.0
* Author: Adnan
* Author URI: http://adnan.pk/
* 
* License: GPLv2+
* 
* Text Domain: wp-analytify-edd
* Domain Path: /languages
* 
*/


if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

define( 'EDD_ANALYTIFY_PLUGIN_DIR',  dirname( __FILE__ ) );

add_action( 'plugins_loaded' , 'analytify_edd_instance', 20 );


function analytify_edd_instance() {

	// check for analytify
	if ( ! file_exists( WP_PLUGIN_DIR . '/wp-analytify-pro/wp-analytify-pro.php' ) ) {
		add_action( 'admin_notices' , 'wpa_edd_install_analytify_pro' );
		return;
	}

	if ( ! class_exists( 'WP_Analytify_Pro_Base' ) ) {
		add_action( 'admin_notices' ,  'wpa_edd_active_analytify_pro' );
		return;
	}

	// Check for edd.
	if ( ! file_exists( WP_PLUGIN_DIR . '/easy-digital-downloads/easy-digital-downloads.php' ) ) {
		add_action( 'admin_notices' , 'install_edd' );
		return;
	}

	if ( ! class_exists( 'Easy_Digital_Downloads' ) ) {
		add_action( 'admin_notices' ,  'active_edd' );
		return;
	}

	include_once EDD_ANALYTIFY_PLUGIN_DIR . '/wp-analytify-edd-class.php';
	new WP_Analytify_Edd();

}


function wpa_edd_install_analytify_pro() {
	$class   = 'notice notice-error is-dismissible';
	$message = __( 'oops! Please Install Analytify Pro.' , 'wp-analytify-edd' );

	printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
}

function wpa_edd_active_analytify_pro() {
	$class   = 'notice notice-error is-dismissible';
	$message = __( 'oops! Please active Analytify Pro.' , 'wp-analytify-edd' );

	printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
}

function install_edd() {
	$class   = 'notice notice-error is-dismissible';
	$message = __( 'oops! Please Install Easy Digital Downloads to enable tracking by Analytify.' , 'wp-analytify-edd' );

	printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
}

function active_edd() {
	$class   = 'notice notice-error is-dismissible';
	$message = __( 'Analytify for Easy Digital Downloads works with Easy Digital Downloads plugin, Please activate Easy Digital Downloads' , 'wp-analytify-edd' );

	printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
}


/**
 * Load TextDoamin
 *
 * @since 1.0.2
 */
function wp_analytify_edd_load_text_domain() {
	$plugin_dir = basename( dirname( __FILE__ ) );
	load_plugin_textdomain( 'wp-analytify-edd', false , $plugin_dir . '/languages/' );
}
add_action( 'init', 'wp_analytify_edd_load_text_domain' );


/**
 * Helper function for translation.
 */
if ( ! function_exists( 'analytify__' ) ) {
	/**
	 * Wrapper for __() gettext function.
	 * @param  string $string     Translatable text string
	 * @param  string $textdomain Text domain, default: wp-analytify
	 * @return void
	 */
	function analytify__( $string, $textdomain = 'wp-analytify' ) {
		return __( $string, $textdomain );
	}
}
if ( ! function_exists( 'analytify_e' ) ) {
	/**
	 * Wrapper for _e() gettext function.
	 * @param  string $string     Translatable text string
	 * @param  string $textdomain Text domain, default: wp-analytify
	 * @return void
	 */
	function analytify_e( $string, $textdomain = 'wp-analytify' ) {
		echo __( $string, $textdomain );
	}
}
